package org.arkerthan.sanityCheck;

/**
 * @author Arkerthan
 */
public class SyntaxError extends Exception {
	private String file;
	private int line, position;
	private String description;
	private int change; //see Element for values; -2 means not thrown
	private boolean warning = false;

	/**
	 * @param description description of error
	 * @param change	  state change as specified in Element
	 */
	public SyntaxError(String description, int change) {
		this.description = description;
		this.change = change;
	}

	/**
	 * @param description description of error
	 * @param change	  state change as specified in Element
	 * @param warning	 whether it is a warning or an error
	 */
	public SyntaxError(String description, int change, boolean warning) {
		this(description, change);
		this.warning = warning;
	}

	/**
	 * @param file at which the error occurred
	 */
	public void setFile(String file) {
		this.file = file;
	}

	/**
	 * @param line in which the error occurred
	 */
	public void setLine(int line) {
		this.line = line;
	}

	/**
	 * @param position at which the error occurred
	 */
	public void setPosition(int position) {
		this.position = position;
	}

	/**
	 * @return error message
	 */
	public String getError() {
		String s = warning ? "Warning: " : "Error: ";
		return s + file + ": " + line + ":" + position + " : " + description;
	}

	/**
	 * @return change that happened in Element before it was thrown. -1 if not thrown.
	 */
	public int getChange() {
		return change;
	}
}
