package org.arkerthan.sanityCheck.element;

import org.arkerthan.sanityCheck.DisallowedTagException;
import org.arkerthan.sanityCheck.UnknownStateException;

import java.util.Arrays;
import java.util.List;

/**
 * @author Arkerthan
 */
public class KnownLogicElement extends KnownElement {
	private static final List<String> allowedTags = Arrays.asList("if", "elseif", "else");
	private final int state;
	private boolean last;
	/*
	0 - if
	1 - elseif
	2 - else
	3 - switch
	4 - case
	5 - default
	 */

	public KnownLogicElement(int line, int pos, String tag, boolean last) {
		this(line, pos, tag);
		this.last = last;
	}

	public KnownLogicElement(int line, int pos, String tag) {
		super(line, pos);
		switch (tag) {
			case "if":
				state = 0;
				break;
			case "elseif":
				state = 1;
				break;
			case "else":
				state = 2;
				break;
			case "switch":
				state = 3;
				break;
			case "case":
				state = 4;
				break;
			case "default":
				state = 5;
				break;
			default:
				throw new DisallowedTagException(tag);
		}
		last = false;
	}

	@Override
	public boolean isOpening() {
		return !last;
	}

	@Override
	public boolean isClosing() {
		return (state != 0 && state != 3) || last;
	}

	@Override
	public boolean isMatchingElement(KnownElement k) {
		if (!(k instanceof KnownLogicElement)) {
			return false;
		}
		KnownLogicElement l = (KnownLogicElement) k;
		switch (state) {
			case 0:
			case 1:
				return l.state == 1 || l.state == 2 || (l.state == 0 && l.last);
			case 2:
				return l.state == 0 && l.last;
			case 3:
			case 4:
				return l.state == 3 || l.state == 4;
			case 5:
				return l.state == 3 && l.last;
			default:
				throw new UnknownStateException(state);
		}
	}

	@Override
	public String getShortDescription() {
		StringBuilder builder = new StringBuilder();
		builder.append(getPositionAsString()).append(" <<");
		if (last) {
			builder.append('/');
		}
		switch (state) {
			case 0:
				builder.append("if");
				break;
			case 1:
				builder.append("elseif");
				break;
			case 2:
				builder.append("else");
				break;
			case 3:
				builder.append("switch");
				break;
			case 4:
				builder.append("case");
				break;
			case 5:
				builder.append("default");
				break;
			default:
				throw new UnknownStateException(state);
		}
		return builder.append(">>").toString();
	}
}
