window.FutureSocieties = (function() {
	return {
		remove: removeFS,
		DecorationCleanup: DecorationCleanup
	};

	// call as FutureSocieties.remove(FS)
	// FS must be a string (e.g. "FSPaternalist" or "FSDegradationist").
	function removeFS(FS) {
		const V = State.variables;
		const arcology = V.arcologies[0];
		const FSDecoration = `${FS }Decoration`;
		const FSSMR = `${FS }SMR`;
		let FSLaw = `${FS }Law`;
		if (arcology[FS] === undefined) {
			// eslint-disable-next-line no-console
			console.log(`ERROR: bad FS reference, $arcologies[0].${FS} not defined`);
			return;
		}

		if (FS === "FSSupremacist" || FS === "FSSubjugationist")
			FSLaw += "ME";
		if (FS !== "FSNull")
			arcology[FSDecoration] = 20;
		arcology[FS] = "unset";
		switch (FS) {
			case "FSPaternalist":
				arcology[FSLaw] = 0;
				arcology[FSSMR] = 0;
				V.slaveWatch = 0;
				break;
			case "FSDegradationist":
				arcology[FSLaw] = 0;
				arcology[FSSMR] = 0;
				V.liveTargets = 0;
				break;
			case "FSGenderRadicalist":
				arcology.FSGenderRadicalistLawBeauty = 0;
				arcology.FSGenderRadicalistLawFuta = 0;
				break;
			case "FSGenderFundamentalist":
				arcology.FSGenderFundamentalistLawBeauty = 0;
				arcology.FSGenderFundamentalistLawBimbo = 0;
				arcology.FSGenderFundamentalistSMR = 0;
				break;
			case "FSTransformationFetishist":
			case "FSAssetExpansionist":
				arcology[FSSMR] = 0;
				break;
			case "FSPhysicalIdealist":
				arcology.FSPhysicalIdealistLaw = 0;
				arcology.FSPhysicalIdealistSMR = 0;
				arcology.FSPhysicalIdealistStrongFat = 0;
				V.martialSchool = 0;
				break;
			case "FSHedonisticDecadence":
				arcology.FSHedonisticDecadenceLaw = 0;
				arcology.FSHedonisticDecadenceLaw2 = 0;
				arcology.FSHedonisticDecadenceSMR = 0;
				arcology.FSHedonisticDecadenceStrongFat = 0;
				break;
			case "FSChattelReligionist":
				arcology.FSChattelReligionistLaw = 0;
				arcology.FSChattelReligionistSMR = 0;
				arcology.FSChattelReligionistCreed = 0;
				V.subsidyChurch = 0;
				break;
			case "FSRepopulationFocus":
				arcology[FSLaw] = 0;
				arcology[FSSMR] = 0;
				V.universalRulesChildrenBecomeBreeders = 0;
				break;
			case "FSRestart":
				arcology[FSLaw] = 0;
				arcology[FSSMR] = 0;
				V.eliteOfficers = 0;
				V.propOutcome = 0;
				V.failedElite = 0;
				break;
			case "FSNull":
				break;
			default: // all others have one law and one SMR
				arcology[FSLaw] = 0;
				arcology[FSSMR] = 0;
				break;
		}

		DecorationCleanup();
		resetFSCredits();
	}

	/* helper function, not callable */
	function resetFSCredits() {
		const V = State.variables;
		let activeFS = 0;
		for (let i = 0; i < setup.FutureSocieties.length; i++) {
			if (V.arcologies[0][setup.FutureSocieties[i]] > 0) {
				activeFS++;
			}
		}
		if (V.arcologies[0].FSNull > 0) { // possibly recalculate for multiculturalism
			activeFS--;
			if (V.FSCreditCount === 4)
				activeFS += V.arcologies[0].FSNull / 25;
			else if (V.FSCreditCount === 6)
				activeFS += V.arcologies[0].FSNull / 17;
			else if (V.FSCreditCount === 7)
				activeFS += V.arcologies[0].FSNull / 15;
			else
				activeFS += V.arcologies[0].FSNull / 20;
		}
		V.FSCredits = Math.max(Math.trunc(V.FSGotRepCredits - activeFS), 0);
	}

	/* call as FutureSocieties.DecorationCleanup() */
	function DecorationCleanup() {
		ValidateFacilityDecoration("brothelDecoration");
		ValidateFacilityDecoration("dairyDecoration");
		ValidateFacilityDecoration("clubDecoration");
		ValidateFacilityDecoration("servantsQuartersDecoration");
		ValidateFacilityDecoration("schoolroomDecoration");
		ValidateFacilityDecoration("spaDecoration");
		ValidateFacilityDecoration("clinicDecoration");
		ValidateFacilityDecoration("arcadeDecoration");
		ValidateFacilityDecoration("cellblockDecoration");
		ValidateFacilityDecoration("masterSuiteDecoration");
		ValidateFacilityDecoration("nurseryDecoration");
		ValidateFacilityDecoration("farmyardDecoration");
	}

	/* helper function, not callable */
	/* decoration should be passed as "facilityDecoration" in quotes. For example, ValidateFacilityDecoration("brothelDecoration"). The quotes are important, do not pass it as a story variable. */
	function ValidateFacilityDecoration(decoration) {
		const V = State.variables;
		const decorationToFSName = {
			standard: "standard",
			Supremacist: "FSSupremacist",
			Subjugationist: "FSSubjugationist",
			'Gender Radicalist': "FSGenderRadicalist",
			'Gender Fundamentalist': "FSGenderFundamentalist",
			Degradationist: "FSDegradationist",
			Paternalist: "FSPaternalist",
			'Body Purist': "FSBodyPurist",
			'Transformation Fetishist': "FSTransformationFetishist",
			'Youth Preferentialist': "FSYouthPreferentialist",
			'Maturity Preferentialist': "FSMaturityPreferentialist",
			'Slimness Enthusiast': "FSSlimnessEnthusiast",
			'Asset Expansionist': "FSAssetExpansionist",
			Pastoralist: "FSPastoralist",
			'Physical Idealist': "FSPhysicalIdealist",
			Hedonistic: "FSHedonisticDecadence",
			'Chattel Religionist': "FSChattelReligionist",
			'Roman Revivalist': "FSRomanRevivalist",
			'Egyptian Revivalist': "FSEgyptianRevivalist",
			'Edo Revivalist': "FSEdoRevivalist",
			'Arabian Revivalist': "FSArabianRevivalist",
			'Chinese Revivalist': "FSChineseRevivalist",
			'Aztec Revivalist': "FSAztecRevivalist",
			'Repopulation Focus': "FSRepopulationFocus",
			Eugenics: "FSRestart"
		};
		const activeFS = decorationToFSName[V[decoration]];

		if (activeFS === "standard") {
			// nothing to do
		} else if (activeFS === undefined) {
			// eslint-disable-next-line no-console
			console.log(`Error: $${decoration} is ${V[decoration]}`);
			V[decoration] = "standard";
		} else if (!Number.isFinite(V.arcologies[0][activeFS])) {
			if (V.arcologies[0][activeFS] !== "unset") {
				// eslint-disable-next-line no-console
				console.log(`Error: $arcologies[0].${activeFS} is ${V.arcologies[0][activeFS]}`);
			}
			V[decoration] = "standard";
		}
	}
})();

window.FSChange = function FSChange(FS, magnitude, bonusMultiplier = 1) {
	const V = State.variables;
	let errorMessage = '';

	switch (FS) {
		case 'Supremacist':
			if (Number.isFinite(V.arcologies[0].FSSupremacist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSSupremacist / V.FSLockinLevel) / 3, 'futureSocieties'); // Reducing the reputation impact of slaves that are not adhering to societal ideals properly
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSSupremacist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSSupremacist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'Subjugationist':
			if (Number.isFinite(V.arcologies[0].FSSubjugationist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSSubjugationist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSSubjugationist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSSubjugationist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'GenderRadicalist':
			if (Number.isFinite(V.arcologies[0].FSGenderRadicalist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSGenderRadicalist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSGenderRadicalist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSGenderRadicalist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'GenderFundamentalist':
			if (Number.isFinite(V.arcologies[0].FSGenderFundamentalist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSGenderFundamentalist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSGenderFundamentalist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSGenderFundamentalist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'Paternalist':
			if (Number.isFinite(V.arcologies[0].FSPaternalist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSPaternalist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSPaternalist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSPaternalist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'Degradationist':
			if (Number.isFinite(V.arcologies[0].FSDegradationist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSDegradationist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSDegradationist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSDegradationist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'AssetExpansionist':
			if (Number.isFinite(V.arcologies[0].FSAssetExpansionist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSAssetExpansionist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSAssetExpansionist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSAssetExpansionist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'SlimnessEnthusiast':
			if (Number.isFinite(V.arcologies[0].FSSlimnessEnthusiast)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSSlimnessEnthusiast / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSSlimnessEnthusiast / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSSlimnessEnthusiast += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'TransformationFetishist':
			if (Number.isFinite(V.arcologies[0].FSTransformationFetishist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSTransformationFetishist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSTransformationFetishist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSTransformationFetishist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'BodyPurist':
			if (Number.isFinite(V.arcologies[0].FSBodyPurist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSBodyPurist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSBodyPurist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSBodyPurist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'MaturityPreferentialist':
			if (Number.isFinite(V.arcologies[0].FSMaturityPreferentialist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSMaturityPreferentialist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSMaturityPreferentialist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSMaturityPreferentialist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'YouthPreferentialist':
			if (Number.isFinite(V.arcologies[0].FSYouthPreferentialist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSYouthPreferentialist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSYouthPreferentialist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSYouthPreferentialist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'Pastoralist':
			if (Number.isFinite(V.arcologies[0].FSPastoralist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSPastoralist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSPastoralist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSPastoralist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'PhysicalIdealist':
			if (Number.isFinite(V.arcologies[0].FSPhysicalIdealist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSPhysicalIdealist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSPhysicalIdealist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSPhysicalIdealist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'ChattelReligionist':
			if (Number.isFinite(V.arcologies[0].FSChattelReligionist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSChattelReligionist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSChattelReligionist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSChattelReligionist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'RomanRevivalist':
			if (Number.isFinite(V.arcologies[0].FSRomanRevivalist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSRomanRevivalist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSRomanRevivalist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSRomanRevivalist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'AztecRevivalist':
			if (Number.isFinite(V.activeArcology.FSAztecRevivalist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSAztecRevivalist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSAztecRevivalist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSAztecRevivalist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'EgyptianRevivalist':
			if (Number.isFinite(V.arcologies[0].FSEgyptianRevivalist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSEgyptianRevivalist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSEgyptianRevivalist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSEgyptianRevivalist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'EdoRevivalist':
			if (Number.isFinite(V.arcologies[0].FSEdoRevivalist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSEdoRevivalist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSEdoRevivalist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSEdoRevivalist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'ArabianRevivalist':
			if (Number.isFinite(V.arcologies[0].FSArabianRevivalist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSArabianRevivalist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSArabianRevivalist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSArabianRevivalist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'ChineseRevivalist':
			if (Number.isFinite(V.arcologies[0].FSChineseRevivalist)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSChineseRevivalist / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSChineseRevivalist / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSChineseRevivalist += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'Repopulationist':
			if (Number.isFinite(V.arcologies[0].FSRepopulationFocus)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSRepopulationFocus / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSRepopulationFocus / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSRepopulationFocus += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'Eugenics':
			if (Number.isFinite(V.arcologies[0].FSRestart)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSRestart / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSRestart / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSRestart += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		case 'Hedonism':
			if (Number.isFinite(V.arcologies[0].FSHedonisticDecadence)) {
				if (magnitude < 0) {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSHedonisticDecadence / V.FSLockinLevel) / 3, 'futureSocieties');
				} else {
					repX(magnitude * V.FSSingleSlaveRep * (V.arcologies[0].FSHedonisticDecadence / V.FSLockinLevel), 'futureSocieties');
				}
				V.arcologies[0].FSHedonisticDecadence += 0.05 * magnitude * V.FSSingleSlaveRep * bonusMultiplier;
			}
			break;
		default:
			errorMessage += `<span class="red">ERROR: bad FS reference</span>`;
	}
	return errorMessage;
};

window.FSChangePorn = function FSChangePorn(FS, magnitude) {
	return FSChange(FS, magnitude, State.variables.pornFameBonus);
};
